"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const color_1 = require("@heroku-cli/color");
const command_1 = require("@heroku-cli/command");
const core_1 = require("@oclif/core");
const tsheredoc_1 = require("tsheredoc");
const url_1 = require("url");
const util_1 = require("util");
const getStdin = require("get-stdin");
const edit = require('edit-string');
class API extends command_1.Command {
    async run() {
        const { args, flags } = await this.parse(API);
        const getBody = async () => {
            const body = flags.body || await getStdin();
            if (!body) {
                this.warn('no stdin provided');
                return;
            }
            try {
                return JSON.parse(body);
            }
            catch (_a) {
                const err = new Error(`Request body must be valid JSON
  Received:
  ${(0, util_1.inspect)(body)}`);
                if (process.stdin.isTTY) {
                    this.warn(err);
                    return JSON.parse(await edit(body, { postfix: '.json' }));
                }
                throw err;
            }
        };
        const request = { headers: {} };
        let { method, path } = args;
        request.method = method;
        if (!path) {
            path = request.method;
            request.method = 'GET';
        }
        request.method = request.method.toUpperCase();
        const uri = this.parseURL(path);
        const version = flags.version || '3';
        request.headers.accept = `application/vnd.heroku+json; version=${version}`;
        if (flags['accept-inclusion']) {
            request.headers['Accept-Inclusion'] = flags['accept-inclusion'];
        }
        if (request.method === 'PATCH' || request.method === 'PUT' || request.method === 'POST') {
            request.body = await getBody();
        }
        const fetch = async (body = []) => {
            core_1.ux.action.start((0, color_1.color) `{cyanBright ${request.method}} ${uri.host}${uri.pathname}`);
            let response;
            try {
                response = await this.heroku.request(uri.toString(), request);
            }
            catch (error) {
                if (!error.http || !error.http.statusCode)
                    throw error;
                core_1.ux.action.stop((0, color_1.color) `{redBright ${error.http.statusCode}}`);
                throw error;
            }
            let msg = (0, color_1.color) `{greenBright ${response.response.statusCode.toString()}}`;
            if (Array.isArray(response.body))
                msg += ` ${response.body.length + body.length} items`;
            core_1.ux.action.stop(msg);
            if (Array.isArray(response.body) && response.response.headers['next-range']) {
                request.headers.range = response.response.headers['next-range'];
                return fetch([...body, ...response.body]);
            }
            return Array.isArray(response.body) ? [...body, ...response.body] : response.body;
        };
        const body = await fetch();
        if (typeof body === 'string') {
            core_1.ux.log(body);
        }
        else {
            core_1.ux.styledJSON(body);
        }
    }
    parseURL(path) {
        try {
            return new url_1.URL(path);
        }
        catch (_a) {
            const normalizedPath = path.startsWith('/') ? path : `/${path}`;
            const protocol = this.heroku.defaults.protocol || 'https:';
            const normalizedURL = `${protocol}//${this.heroku.defaults.host}${normalizedPath}`;
            return new url_1.URL(normalizedURL);
        }
    }
}
exports.default = API;
API.args = {
    method: core_1.Args.string({
        description: 'GET, POST, PUT, PATCH, or DELETE',
        required: true,
    }),
    path: core_1.Args.string({
        description: 'endpoint to call',
    }),
};
API.description = `make a manual API request
The api command is a convenient but low-level way to send requests
to the Heroku API. It sends an HTTP request to the Heroku API
using the given method on the given path. For methods PUT, PATCH,
and POST, it uses stdin unmodified as the request body. It prints
the response unmodified on stdout.

It is essentially like curl for the Heroku API.

Method name input will be upcased, so both 'heroku api GET /apps' and
'heroku api get /apps' are valid commands.`;
API.examples = [(0, tsheredoc_1.default) `
    $ heroku api GET /apps/myapp
    {
      created_at: "2011-11-11T04:17:13-00:00",
      id: "12345678-9abc-def0-1234-456789012345",
      name: "myapp",
      …
    }
  `, (0, tsheredoc_1.default) `
    $ heroku api PATCH /apps/myapp/config-vars --body '{"FOO": "bar"}'
    {
      FOO: "bar"
      …
    }
    `, (0, tsheredoc_1.default) `
    $ printf '{"updates":[{"type":"web", "quantity":2}]}' | heroku api POST /apps/myapp/formation
    [
      {
        "app": {
          "name": "myapp",
          "id": "01234567-89ab-cdef-0123-456789abcdef"
        },
        "quantity": 2,
        "type": "web",
        "updated_at": "2012-01-01T12:00:00Z"
        ...
      }
    ]
    `];
API.flags = {
    'accept-inclusion': command_1.flags.string({ char: 'a', description: 'Accept-Inclusion header to use' }),
    body: command_1.flags.string({ char: 'b', description: 'JSON input body' }),
    version: command_1.flags.string({ char: 'v', description: 'version to use (e.g. 2, 3, or 3.variant)' }),
};
